@@ -382,8 +382,7 @@ gdk_pixbuf_copy (const GdkPixbuf *pixbuf)
 	 * rowstride?
 	 */
 
-	size = ((pixbuf->height - 1) * pixbuf->rowstride +
-		pixbuf->width * ((pixbuf->n_channels * pixbuf->bits_per_sample + 7) / 8));
+	size = gdk_pixbuf_get_byte_length (pixbuf);
 
 	buf = g_try_malloc (size * sizeof (guchar));
 	if (!buf)
@@ -528,9 +527,9 @@ gdk_pixbuf_get_bits_per_sample (const GdkPixbuf *pixbuf)
  *
  * Queries a pointer to the pixel data of a pixbuf.
  *
- * Return value: A pointer to the pixbuf's pixel data.  Please see <xref linkend="image-data"/>
- * for information about how the pixel data is stored in
- * memory.
+ * Return value: (array): A pointer to the pixbuf's pixel data.
+ * Please see <xref linkend="image-data"/> for information about how
+ * the pixel data is stored in memory.
  **/
 guchar *
 gdk_pixbuf_get_pixels (const GdkPixbuf *pixbuf)
@@ -540,6 +539,34 @@ gdk_pixbuf_get_pixels (const GdkPixbuf *pixbuf)
 	return pixbuf->pixels;
 }
 
+/**
+ * gdk_pixbuf_get_pixels_with_length:
+ * @pixbuf: A pixbuf.
+ * @length: (out): The length of the binary data.
+ *
+ * Queries a pointer to the pixel data of a pixbuf.
+ *
+ * Return value: (array length=length): A pointer to the pixbuf's
+ * pixel data.  Please see <xref linkend="image-data"/>
+ * for information about how the pixel data is stored in
+ * memory.
+ *
+ * Rename to: gdk_pixbuf_get_pixels
+ *
+ * Since: 2.26
+ */
+guchar *
+gdk_pixbuf_get_pixels_with_length (const GdkPixbuf *pixbuf,
+                                   guint           *length)
+{
+	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
+
+        if (length)
+                *length = gdk_pixbuf_get_byte_length (pixbuf);
+
+	return pixbuf->pixels;
+}
+
 /**
  * gdk_pixbuf_get_width:
  * @pixbuf: A pixbuf.
@@ -589,6 +616,25 @@ gdk_pixbuf_get_rowstride (const GdkPixbuf *pixbuf)
 	return pixbuf->rowstride;
 }
 
+/**
+ * gdk_pixbuf_get_byte_length:
+ * @pixbuf: A pixbuf
+ *
+ * Returns the length of the pixel data, in bytes.
+ *
+ * Return value: The length of the pixel data.
+ *
+ * Since: 2.26
+ */
+gsize
+gdk_pixbuf_get_byte_length (const GdkPixbuf *pixbuf)
+{
+	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), -1);
+
+        return ((pixbuf->height - 1) * pixbuf->rowstride +
+                pixbuf->width * ((pixbuf->n_channels * pixbuf->bits_per_sample + 7) / 8));
+}
+
 
 
 /* General initialization hooks */
@@ -690,7 +736,7 @@ gdk_pixbuf_fill (GdkPixbuf *pixbuf,
  * Return value: the value associated with @key. This is a nul-terminated 
  * string that should not be freed or %NULL if @key was not found.
  **/
-G_CONST_RETURN gchar *
+const gchar *
 gdk_pixbuf_get_option (GdkPixbuf   *pixbuf,
                        const gchar *key)
 {
